<?php

namespace TeamChat\SDK;

use TeamChat\SDK\Activities\Activity;

/**
 * Description of ActivityBroker.
 *
 * @author Team Chat <support@teamchat.com>
 */
class ActivityBroker
{
    const ACTIVITY_ALIAS = 'alias';
    const ACTIVITY_BOT_ADDED = 'botadded';
    const ACTIVITY_COMMENT = 'comment';
    const ACTIVITY_EVENT = 'event';
    const ACTIVITY_EVENT_IN_ALIASED_ROOM = 'eventinaliasedroom';
    const ACTIVITY_KEYWORD = 'keyword';
    const ACTIVITY_MEMBER_ADDED = 'memberadded';
    const ACTIVITY_MEMBER_LEFT = 'memberleft';
    const ACTIVITY_MEMBER_REMOVED = 'memberremoved';
    const ACTIVITY_MSG = 'msg';
    const ACTIVITY_WELCOME = 'welcome';

    protected $handlers = array();

    public function onActivity($name, $handler)
    {
        if (!key_exists($name, $this->handlers)) {
            $this->handlers[$name] = array();
        }
        $this->handlers[$name][] = $handler;
    }
    public function getActivityHandlers($name)
    {
        if (!key_exists($name, $this->handlers)) {
            return array();
        }

        return $this->handlers[$name];
    }
    public function consume(Client\IAPIClient $client, Activity $activity, $botid)
    {
        if (is_a($activity, __NAMESPACE__.'\Activities\Message')) {
            /* @var $activity Activities\Message */
            $msgId = $activity->id;
            foreach ($this->getActivityHandlers(self::ACTIVITY_MSG) as $handler) {
                /* @var $handler Handlers\MessageHandler */
                $handler->execute($client);
            }
            foreach ($this->getActivityHandlers(self::ACTIVITY_KEYWORD) as $handler) {
                /* @var $handler Handlers\KeywordHandler */
                if ($handler->matches($activity)) {
                    $handler->execute($client);
                }
            }
            $textObj = json_decode($activity->text);
            if (!empty($textObj)) {
                if (property_exists($textObj, 'formid')) {
                    $formId = $textObj->formid;
                }
                if ($formId != null) {
                    $aliases = $client->udata->getAlias($formId);
                }
                $formtype = $textObj->type;
                $lastAction = $textObj->latestAction;

                if ($lastAction === 'reply' || $formtype === 'locform') {
                    foreach ($this->getActivityHandlers(self::ACTIVITY_ALIAS) as $handler) {
                        /* @var $handler Handlers\AliasHandler */
                        if ($handler->aliasIn($aliases)) {
                            $handler->execute($client);
                        }
                    }
                }
            }
        }
        if (is_a($activity, __NAMESPACE__.'\Activities\Welcome')) {
            foreach ($this->getActivityHandlers(self::ACTIVITY_WELCOME) as $handler) {
                $handler->execute($client);
            }
        }
        if (is_a($activity, __NAMESPACE__.'\Activities\Event')) {
            foreach ($this->getActivityHandlers(self::ACTIVITY_EVENT) as $handler) {
                $handler->execute($client);
            }
            if (is_a($activity, __NAMESPACE__.'\Activities\MemberAdded')) {
                /* @var $activity Activities\MemberAdded */
                foreach ($this->getActivityHandlers(self::ACTIVITY_MEMBER_ADDED) as $handler) {
                    $handler->execute($client);
                }
                if (in_array($botid, $activity->getUserDetails())) {
                    foreach ($this->getActivityHandlers(self::ACTIVITY_BOT_ADDED) as $handler) {
                        $handler->execute($client);
                    }
                }
            }
            if (is_a($activity, __NAMESPACE__.'\Activities\MemberRemoved')) {
                foreach ($this->getActivityHandlers(self::ACTIVITY_MEMBER_REMOVED) as $handler) {
                    $handler->execute($client);
                }
            }
            if (is_a($activity, __NAMESPACE__.'\Activities\MemberLeft')) {
                foreach ($this->getActivityHandlers(self::ACTIVITY_MEMBER_LEFT) as $handler) {
                    $handler->execute($client);
                }
            }
        }
    }
    public function onAlias($callback, $alias)
    {
        $this->onActivity(self::ACTIVITY_ALIAS, new Handlers\AliasHandler($callback, $alias));
    }
    public function onBotAdded($callback)
    {
        $this->onActivity(self::ACTIVITY_BOT_ADDED, new Handlers\BotAddedHandler($callback));
    }
    public function onComment($callback, $comment)
    {
        $this->onActivity(self::ACTIVITY_COMMENT, new Handlers\CommentHandler($callback, $comment));
    }
    public function onEvent($callback)
    {
        $this->onActivity(self::ACTIVITY_EVENT, new Handlers\EventHandler($callback));
    }
    public function onEventInAliasedRoom($callback, $keyword)
    {
        $this->onActivity(self::ACTIVITY_EVENT_IN_ALIASED_ROOM, new Handlers\EventInAliasedRoomHandler($callback, $keyword));
    }
    public function onKeyword($callback, $keyword, $isCaseSensitive = false, $description = '')
    {
        $this->onActivity(self::ACTIVITY_KEYWORD, new Handlers\KeywordHandler($callback, $keyword, $isCaseSensitive, $description));
    }
    public function onMemberAdded($callback)
    {
        $this->onActivity(self::ACTIVITY_MEMBER_ADDED, new Handlers\MemberAddedHandler($callback));
    }
    public function onMemberLeft($callback)
    {
        $this->onActivity(self::ACTIVITY_MEMBER_LEFT, new Handlers\MemberLeftHandler($callback));
    }
    public function onMemberRemoved($callback)
    {
        $this->onActivity(self::ACTIVITY_MEMBER_REMOVED, new Handlers\MemberRemovedHandler($callback));
    }
    public function onMsg($callback)
    {
        $this->onActivity(self::ACTIVITY_MSG, new Handlers\MessageHandler($callback));
    }
    public function onWelcome($callback)
    {
        $this->onActivity(self::ACTIVITY_WELCOME, new Handlers\WelcomeHandler($callback));
    }
}
