<?php

namespace TeamChat\SDK\Client;

use TeamChat\SDK\ActivityBroker;
use TeamChat\SDK\Activities\ActivityFactory;
use TeamChat\SDK\Data\PersistenceLayerInterface;
use TeamChat\SDK\Data\Record;
use TeamChat\SDK\Messages;
use TeamChat\SDK\Room;
use SplDoublyLinkedList;
use Psr\Log\LoggerInterface;
use Monolog\Logger;
use Monolog\Handler\StreamHandler;

/**
 * Description of APIClient.
 *
 * @author Team Chat <support@teamchat.com>
 */
class APIClient implements IAPIClient
{
    /**
     * @var SplDoublyLinkedList
     */
    public $outQueue;

    /**
     * @var SplDoublyLinkedList
     */
    public $inQueue;

    /**
     * @var \IHTTPAPIClient
     */
    public $httpAPIClient;
    public $broker;
    public $context;

    private $eventStream;

    public $lastEventTimestamp;

    /**
     * @var LoggerInterface
     */
    public $logger;

    /**
     * @var \TeamChat\SDK\Data\UserDataInterface User Data 
     */
    public $udata;

    /**
     * @var \TeamChat\SDK\Data\BotDataInterface Bot Data
     */
    public $bdata;

    public function __construct(IHTTPAPIClient $httpAPIClient, ActivityBroker $broker, PersistenceLayerInterface $layer)
    {
        $this->outQueue = new SplDoublyLinkedList();
        $this->inQueue = new SplDoublyLinkedList();
        $this->httpAPIClient = $httpAPIClient;
        $this->logger = new Logger('TeamChat');
        $this->logger->pushHandler(new StreamHandler('php://stdout', \Psr\Log\LogLevel::INFO));
        $this->broker = $broker;
//        $this->context = new \TeamChat\SDK\Context(null);
        $record = new Record($layer);
        $this->udata = $record;
        $this->bdata = $record;

        $credentials = $this->httpAPIClient->getTokenCredentials();
        $this->udata->updateUserId($credentials->userId);
        $this->udata->updateToken($credentials->token);

        $this->lastEventTimestamp = $this->udata->getLastMsgTimestamp();
        if (is_null($this->lastEventTimestamp)) {
            $this->lastEventTimestamp = round(microtime(true) * 1000);
        }
    }

    /**
     * Main function which handles runing of the bot.
     * First tries to send pending messages, one by one
     * Then processes the received events, one by one
     * Finally gets new events from the server.
     */
    public function run()
    {
        $readDelay = 0;
        while (true) {
            if (!$this->outQueue->isEmpty()) {
                $this->sendNextMessage();
                $readDelay = 0;
            } elseif (!$this->inQueue->isEmpty()) {
                $this->consumeNextEvent();
                $readDelay = 0;
            } else {
                usleep($readDelay);
                $this->getNewEvents();
                $readDelay = 10000; //Make sure we don't eat up 100% CPU. Introduce some delay only between consecutive reads.
            }
        }
    }

    public function setLogger(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }

    public function consumeNextEvent()
    {
        $activity = $this->inQueue->shift();
        $this->context = new \TeamChat\SDK\Context($activity, $this, $this->udata);
        $this->logger->info('Consuming Activity', array('activity' => $activity));
        $this->broker->consume($this, $activity, $this->udata->getUserId());
    }

    public function getNewEvents()
    {
        $events = array();
        while (($event = $this->fetchEvent()) !== false) {
            if ($event !== true) {
                $events[] = $event;
            }
        }
        if (sizeof($events) == 0) {
            return;
        }
        foreach ($events as $event) {
            $this->inQueue->push($event);
            $this->updateLastEventTimestamp($event);
        }
    }

    public function enqueueMessage(Messages\Message $message)
    {
        $this->outQueue->push($message);
    }

    public function sendNextMessage()
    {
        $message = $this->outQueue->shift();
        $this->sendMessage($message);
    }

    public function sendMessage($message)
    {
        $this->logger->info('Trying to send the message');
        $response = $this->httpAPIClient->postMessage($message);
        $this->logger->info('Message Sent', array('response' => $response));

        return $response;
    }

    public function updateLastEventTimestamp($event)
    {
        if (!is_null($event) && isset($event->date)) {
            $this->lastEventTimestamp = $event->date;
            $this->udata->updateLastMsgTimestamp($this->lastEventTimestamp);
            $this->logger->info('Timestamp updated', array('timestamp' => $event->date));
        }
    }

    private function getEventStream()
    {
        $this->logger->debug('Trying to get an Event Stream');
        if (!$this->eventStream || feof($this->eventStream)) {
            $this->logger->info('Creating a new Event Stream');
            $credentials = $this->httpAPIClient->getTokenCredentials();
            $opts = array(
                'http' => array(
                    'method' => 'GET',
                    'header' => 'Accept: application/vnd.gupshup.privatechat-v1+json',
                    'header' => "Authorization: Gupshup {$credentials->userId}:{$credentials->token}:echat",
                    'user_agent' => 'TeamchatPHP/0.1',
                    'protocol_version' => 1.1,
                    'ignore_errors' => true,
                ),
            );
            $context = stream_context_create($opts);
            $this->eventStream = fopen($this->httpAPIClient->getEventsUrl($this->lastEventTimestamp), 'r', false, $context);
            stream_set_blocking($this->eventStream, 0);
        }

        return $this->eventStream;
    }

    private function fetchEvent()
    {
        $data = fgets($this->getEventStream());
        if ($data === false) {
            $this->logger->debug('No event received');

            return false;
        } elseif (substr($data, 0, 5) === 'data:') {
            //Workaround for data sent via multiple TCP packets.
            while (substr($data, -1) !== "\n") {
                $data .= fgets($this->getEventStream());
            }
            $this->logger->info('Event Received', array('data' => $data));

            return ActivityFactory::create(json_decode(substr($data, 5)), $this->udata->getUserId());
        } else {
            $this->logger->warning('Unknown Event Received', array('data' => $data));

            return true;
        }
    }

    /**
     * @return \TeamChat\SDK\ContextInterface
     */
    public function context()
    {
        return $this->context;
    }

    public function getRoomId(Room $room)
    {
        $id = $room->getId();
        if (!is_null($id)) {
            return $id;
        }
        $emails = implode(',', $room->getEmailsToAdd());
        if (empty($emails)) {
            return;
        }
        $name = $room->getName();
        if (empty($name)) {
            $name = 'no-name';
        }
        $response = $this->httpAPIClient->createRoom($name, $emails, $room->welcomeMessage());
        if (isset($response->_id)) {
            $this->logger->info('Created Room', array('response' => $response));
            $room->setId($response->_id);
        }

        return $room->getId();
    }

    public function perform(Room $room)
    {
        $id = $this->getRoomId($room);
        foreach ($room->getMsgs() as $chatlet) {
            /* @var $chatlet Messages\Chatlets\ChatletInterface */
            $message = new \TeamChat\SDK\Messages\Message();
            $message->recipient_id = $id;
            $message->recipient_type = \TeamChat\SDK\Messages\Message::RECIPIENT_ROOM;
            $message->text = $chatlet->raw();
            $this->logger->info($message->text);
            $response = $this->sendMessage($message);
            if (property_exists($response, '_id')) {
                $msgid = $response->_id;
                $this->logger->info("Got the message id as $msgid");
                $this->udata->updateLastMsgId($msgid);
            }
            if (property_exists($response, 'text')) {
                $text = json_decode($response->text);
                if (!empty($text) && property_exists($text, 'formid')) {
                    $formid = $text->formid;
                    $this->logger->info("Got the form id as $formid");
                    $chatlet->setFormId($formid);
                    foreach ($chatlet->getAlias() as $alias) {
                        $this->udata->updateAlias($formid, $alias);
                    }
                }
            }
        }
    }
}
