<?php

namespace TeamChat\SDK\Client;

use TeamChat\SDK\Messages\Message;
use TeamChat\SDK\Events\EventsFactory;
use Buzz\Browser;
use Buzz\Client\Curl;
use stdClass;

/**
 * Description of HTTPAPIClient.
 *
 * @author Team Chat <support@teamchat.com>
 */
class HTTPAPIClient implements IHTTPAPIClient
{
    private $apikey;
    private $token;
    private $userId;

    public function __construct($apikey)
    {
        $this->apikey = $apikey;
        $this->refreshToken();
    }

    public function getBaseUrl()
    {
        return 'http://api.webaroo.com/api';
    }

    public function getEventsSince($timestamp)
    {
        $url = $this->getEventsUrl($timestamp);
        $client = new Browser(new Curl());
        $headers = array(
            'Accept' => 'application/vnd.gupshup.privatechat-v1+json',
            'Authorization' => "Gupshup {$this->userId}:{$this->token}:echat",
        );
        $response = $client->get($url, $headers);
        $entries = explode("\n", $response->getContent());
        $messages = array();
        foreach ($entries as $entry) {
            if (substr($entry, 0, 5) === 'data:') {
                $messages[] = EventsFactory::create(json_decode(substr($entry, 5)));
            }
        }

        return $messages;
    }

    public function postMessage(Message $message)
    {
        switch ($message->recipient_type) {
            case Message::RECIPIENT_P2P:
                return $this->postP2pMessage($message->recipient_id, $message->raw());
            case Message::RECIPIENT_ROOM:
                return $this->postGroupMessage($message->recipient_id, $message->raw());
        }
    }

    public function postP2pMessage($email, $text)
    {
        $url = $this->getP2pMessageUrl();
        $client = new Browser(new Curl());
        $params = array(
            'email' => $email,
            'message' => $text,
        );
        $headers = array(
            'apikey' => $this->apikey,
            'Expect:' => "",
        );
        $response = $client->post($url, $headers, http_build_query($params));

        return json_decode($response->getContent());
    }

    public function postGroupMessage($groupId, $text)
    {
        $url = $this->getGroupMessageUrl();
        $client = new Browser(new Curl());
        $params = array(
            'groupid' => $groupId,
            'message' => $text,
        );
        $headers = array(
            'apikey' => $this->apikey,
            'Expect:' => "",
        );
        $response = $client->post($url, $headers, http_build_query($params));

        return json_decode($response->getContent());
    }

    public function getChatletById($id)
    {
        $url = $this->getChatletUrl($id);
        $client = new Browser(new Curl());
        $headers = array(
            'Accept' => 'application/vnd.gupshup.privatechat-v1+json',
            'Authorization' => "Gupshup {$this->userId}:{$this->token}:echat",
        );
        $response = $client->get($url, $headers);

        return json_decode($response->getContent());
    }
    public function getToken()
    {
        if (is_null($this->token)) {
            $this->refreshToken();
        }

        return $this->token;
    }

    public function getTokenCredentials()
    {
        if (is_null($this->token) || is_null($this->userId)) {
            $this->refreshToken();
        }
        $credentials = new stdClass();
        $credentials->token = $this->token;
        $credentials->userId = $this->userId;

        return $credentials;
    }

    public function refreshToken()
    {
        $url = 'http://api.webaroo.com/GatewayAPI2/rest/login';
        $client = new Browser(new Curl());
        $headers = array(
            'apikey' => $this->apikey,
            'Expect:' => "",
        );
        $response = $client->post($url, $headers);
        $login = json_decode($response->getContent());
        $this->token = $login->token->token;
        $this->userId = $login->profile->userId;
    }

    public function createRoom($name, $emails, $welcomeMessage)
    {
        $url = $this->getRoomsUrl();
        $client = new Browser(new Curl());
        $params = array(
            'name' => $name,
            'emails' => $emails,
        );
        if (!empty($welcomeMessage)) {
            $params['welcomeMsg'] = $welcomeMessage;
        }
        $headers = array(
            'apikey' => $this->apikey,
            'Expect:' => "",
        );
        $response = $client->post($url, $headers, http_build_query($params));

        return json_decode($response->getContent());
    }

    //Get various URL endpoints
    public function getEventsUrl($after = null)
    {
        $query = '';
        if (!is_null($after)) {
            $params = array(
                'after' => $after,
            );
            $query = '?'.http_build_query($params);
        }

        return $this->getBaseUrl().'/events'.$query;
    }

    public function getGroupMessageUrl()
    {
        return 'http://api.webaroo.com/GatewayAPI2/rest/message';
    }

    public function getP2pMessageUrl()
    {
        return 'http://api.webaroo.com/GatewayAPI2/rest/pmessage';
    }

    public function getRoomsUrl()
    {
        return 'http://api.webaroo.com/GatewayAPI2/rest/rooms/private';
    }
    public function getChatletUrl($id)
    {
        return $this->getBaseUrl()."/form/$id";
    }
}
